--
-- GameExtension
--
-- @author:    	Xentro (Marcus@Xentro.se)
-- @website:	www.Xentro.se
-- @history:	
-- 

GameExtension = {};
GameExtension.version 		  = ModsUtil.findModItemByModName(g_currentModName).version;
GameExtension.modName 		  = g_currentModName;
GameExtension.modDirectory    = g_currentModDirectory;
GameExtension.hudElements 	  = Utils.getFilename("hudElements.dds", GameExtension.modDirectory .. "huds/");

GameExtension.serverFilename  = "GameExtension_Server.xml";
GameExtension.clientFilename  = "GameExtension_Client.xml";

GameExtension.classOverrides  = {}; -- Delay our overrides so that other mods can alter there behaviour

GameExtension.modules 		  = {};
GameExtension.specializations = {};

GameExtension.colorCodes      = {
	BG_GREY_1		= {0.0284, 0.0284, 0.0284, 1}, -- R, G, B, Alpha
	BG_BRIGHT_BLACK = {0.0075, 0.0075, 0.0075, 1},
	
	TEXT_GREEN		= {0.2122, 0.5271, 0.0307, 1},
	TEXT_ORANGE		= {1.0000, 0.4910, 0.0000, 1},
	TEXT_YELLOW		= {0.9010, 0.8390, 0.1560, 1},
	TEXT_RED		= {0.8069, 0.0097, 0.0097, 1},
	TEXT_LIGHT_BLUE	= {0.8069, 0.0097, 0.0097, 1},
	
	ICON_GREY		= {0.1900, 0.1900, 0.1900, 1},
	ICON_ORANGE		= {0.9046, 0.2874, 0.0123, 1},
	ICON_GREEN		= {0.2122, 0.5271, 0.0307, 1},
	ICON_RED		= {0.7843, 0.0000, 0.0000, 1}
};


-- Debuging --
GameExtension.ERROR 		= 0;
GameExtension.WARNING 		= 1;
GameExtension.NOTICE 		= 2;
GameExtension.DEBUG 		= 3;
GameExtension.MESSAGE_MODE 	= GameExtension.NOTICE;


-- Code Activations.. Better word?
GameExtension.HIDDEN_LOGIN_PASSWORD = true;


-- Utils Files --
source(Utils.getFilename("GameExtensionUtils.lua", GameExtension.modDirectory));

-- Items we dont want other parts to mess with
GameExtension.addBlackListItem("ALLOW_DEACTIVATE");
GameExtension.addBlackListItem("DEACTIVATE_ON_MP");
GameExtension.addBlackListItem("class");
GameExtension.addBlackListItem("sortedKeys");
GameExtension.addBlackListItem("isSpec");


-- We use this as g_currentMission is unreliable for deactivating modules
-- We need to make sure the name always stay the same.
if GameExtension.modName == "FS17_GameExtension" then
	-- Load Default Settings -- 
	GameExtension.loadSettingXML(GameExtension.modDirectory .. "defaultSettings.xml", false, true, true);


	-- Source Files --
	source(Utils.getFilename("events/SettingEvent.lua", 	  GameExtension.modDirectory));
	source(Utils.getFilename("events/SynchSettingsEvent.lua", GameExtension.modDirectory));
	source(Utils.getFilename("gui/GameExtensionGUI.lua", 	  GameExtension.modDirectory));

	addModEventListener(GameExtension);
	
	
	-- Add the GameExtension Specialization
	source(Utils.getFilename("AddSpecialization.lua", 	  	  GameExtension.modDirectory));
end;

function GameExtension:loadMap()
	-- Game is reset on game exit, no need to limit this to once as its already done.
	
	-- Testing purpose to confirm it over many more players
	if g_currentMission.GameExtension == nil then
		g_currentMission.GameExtension = self;
	else
		log("ERROR", "This mod was loaded more then 1 time, contact Xentro about this find!");
		return;
	end;
	
	-- If mod is active then show setting, MOD, MODULE, SETTING, STATE (Optional)
	GameExtension.showSettingForMod("ZZZ_driveControl", "VEHICLE", "DC_USE_SHUTTLE");
	GameExtension.showSettingForMod("FS17_GearboxAddon", "VEHICLE", "GB_USE_GEARBOX");

	if g_dedicatedServerInfo == nil then
		GameExtension.showSettingForMod(GameExtension.modName, "TIME", "AUTO_SAVE_INTERVAL");
	end;
	
	if g_currentMission.missionDynamicInfo.isMultiplayer then
		GameExtension.showSettingForMod(GameExtension.modName, "PLAYER", "CHAINSAW_ADMIN_ONLY");
	end;
	
	
	for moduleName, v in pairs(GameExtension.modules) do
		if GameExtension.getSetting(moduleName) then
			if g_currentMission.missionDynamicInfo.isMultiplayer and GameExtension.getSetting(moduleName, "DEACTIVATE_ON_MP") then
				GameExtension.deactivateModule(moduleName, GameExtension.modName .. ". Module aren't allowed to be used in an multiplayer game.");
			else
				local fileName = Utils.getFilename("modules/" .. moduleName .. ".lua", GameExtension.modDirectory);
				
				if fileExists(fileName) then
					source(fileName);
					-- log("DEBUG", moduleName .. ".lua is loaded.");
				end;
				
				
				loadstring("moduleClass=" .. moduleName)();
				if moduleClass ~= nil then
					v.class = moduleClass;
					
					if v.isSpec then
						GameExtension.addSpecialization(moduleName, v.class);
					else
						if v.class.loadMap ~= nil then
							v.class.loadMap(self);
						end;
					end;
					
					-- call setting function if any..
					for setting, s in pairs(v) do
						if type(s) == "table" and s.func ~= nil then
							v.class[s.func](self, s.value);
						end;
					end;
				end;
			end;
		end;
	end;
	
	if AddSpecialization ~= nil then
		AddSpecialization:add();
	end;
	
	-- Load server and client data seperate
	local oldFile = Utils.getFilename("GameExtension.xml", getUserProfileAppPath());
	
	if fileExists(oldFile) then
		log("NOTICE", "Found old format of saved data, converting to new.");
		GameExtension.loadSettingXML(oldFile, true, false); -- dont set module state..
		
		if g_currentMission:getIsServer() then
			GameExtension.saveFile(GameExtension.serverFilename, true);
		end;
		
		if g_currentMission:getIsClient() and g_dedicatedServerInfo == nil then -- dont load client settings for dedicated servers
			GameExtension.saveFile(GameExtension.clientFilename, false);
		end;
	
		getfenv(0)["deleteFile"](oldFile); -- Remove old file
	else
		if g_currentMission:getIsServer() then
			GameExtension.loadSettingXML(Utils.getFilename(GameExtension.serverFilename, getUserProfileAppPath()), true, true);
		end;
		
		if g_currentMission:getIsClient() and g_dedicatedServerInfo == nil then -- dont load client settings for dedicated servers
			GameExtension.loadSettingXML(Utils.getFilename(GameExtension.clientFilename, getUserProfileAppPath()), true, false);
		end;
	end;
	
	-- GUI Stuff
	g_currentMission.GameExtension_GUI_isOpen = false; -- Some sort of fix for dialog..
	getfenv(0)["g_gameExtensionGUI"] = GameExtensionGUI:new();
	g_gui:loadGui(GameExtension.modDirectory .. "gui/GameExtension.xml", "gameExtensionGUI", g_gameExtensionGUI);
	
	self.firstTimeRun = false;
	self.tickRate = {current = 0, limit = 30};
end;

function GameExtension:deleteMap()
	GameExtension.callFunc(self, "deleteMap", {});
	
	g_gameExtensionGUI:delete();
	self.finishLoadingGUI = false;
end;

function GameExtension:mouseEvent(posX, posY, isDown, isUp, button)
	-- GameExtension.callFunc(self, "mouseEvent", {posX, posY, isDown, isUp, button});
end;

function GameExtension:keyEvent(unicode, sym, modifier, isDown)
	-- GameExtension.callFunc(self, "keyEvent", {unicode, sym, modifier, isDown});
end;

function GameExtension:update(dt)
	if self.firstTimeRun then
		if not self.finishLoadingGUI then
			self.finishLoadingGUI = true;
			g_gameExtensionGUI:load();
		end;
	else
		-- Give the option of stopping overrides and we will have less conflicts with other mods this way seeing we are last on the ball so to speak
		-- Preferable set this in the loadMap function, any later will render it useless.
		--[[ Example
		local modName = "FS17_GameExtension";
		if g_modIsLoaded[modName] then
			_G[modName]["GameExtension"]["OVERRIDE_VEHICLE_FILLLEVEL"] = true;
		end;
		]]
		for name, v in pairs(GameExtension.classOverrides) do
			if not GameExtension[name] then
				v.class[v.func] = Utils.overwrittenFunction(v.class[v.func], v.new);
			else
				log("NOTICE", "Override for " .. name .. " have been successfully stopped.");
			end;
		end;
	
		-- Finish custom mod settings
		if GameExtension.tempShowSetting ~= nil then
			for _, v in pairs(GameExtension.tempShowSetting) do
				if g_modIsLoaded[v.modName] then
					GameExtension.modules[v.moduleName][v.setting].showInSettings = v.state;
				end;
			end;
			
			GameExtension.tempShowSetting = nil;
		end;
		
		-- Custom pages must be added before gui has finished loading so its adviced to do it on the 1th update in the update function, on 2nd update the loading will finish.
		--[[
		-- Only call this once!
		if g_gameExtensionGUI ~= nil then
			local entry = {};
			
			entry.pageName = "Misc - Custom Page";
			entry.isAdminPage = false;
			entry.settings = {};
			
			entry.settings[1] = {
				name = "Testing 1",
				toolTip = "Our Epic tooltip!",
				parent = self, 
				variableName = "firstTimeRun",
				func = self.testFunction,
				type = "bool", -- float, int, bool
				
				showInSettings = true,		-- I use this to toggle an setting on / off depending if mod X is being used. 
				isLocked = false,			-- Leave this alone
				isLockedByForce = false,	-- Use this to disable setting
				limits = {min = 0, max = 1} -- for floats or int add this
			};
			
			-- Create an new page
			g_gameExtensionGUI:addPage("Misc", entry);
			
			-- Use this if you want to add settings to an already existing page
			g_gameExtensionGUI:addSettingsToPage("Misc", {entry.settings[1], entry.settings[1]});
		end;
		]]
	end;
	
	if g_currentMission:getIsClient() then
		if self:getIsActiveForInput() then
			if not g_currentMission.currentTipTriggerIsAllowed then
				local allow = true;
				
				if g_currentMission.controlledVehicle ~= nil then
					local root = g_currentMission.controlledVehicle;
					
					if root.selectedImplement ~= nil and root.selectedImplement.object.overloading ~= nil then
						if root.selectedImplement.object.overloading.canToggleOverloading then
							allow = InputBinding.getHasInputConflict(InputBinding.TOGGLE_GUI_SCREEN, root.selectedImplement.object.conflictCheckedInputs)
							
							-- Just to be safe that it really arent allowed
							if not allow then
								allow = not root.selectedImplement.object:getIsOverloadingAllowed();
							end;
						end;
					end;
				end;
				
				if InputBinding.hasEvent(InputBinding.TOGGLE_GUI_SCREEN, true) and allow then
					g_gui:showGui("gameExtensionGUI");
				end;
			end;
		end;
	end;
	
	GameExtension.callFunc(self, "update", {dt});
	
	if self.tickRate.current >= self.tickRate.limit then
		GameExtension.callFunc(self, "updateTick", {dt});
		self.tickRate.current = 0;
	else
		self.tickRate.current = self.tickRate.current + 1;
	end;
	
	self.firstTimeRun = true;
	
	if GameExtension.MESSAGE_MODE == GameExtension.DEBUG then
		local strs;
		
		for name, v in pairs(GameExtension.visualDebug) do
			if strs == nil then
				strs = {"Name\n", "Value\n"};
			end;
			
			strs[1] = strs[1] .. string.format("%s\n", name);
			
			if type(v.parent) == "table" then
				if v.parent[v.variable] ~= nil then
					strs[2] = strs[2] .. string.format(": %s\n", tostring(v.parent[v.variable]));
				else
					strs[2] = strs[2] .. string.format(": %s\n", "----");
				end;
			else
				strs[2] = strs[2] .. string.format(": %s\n", tostring(v.parent));
			end;
		end;
		
		if strs ~= nil then
			setTextColor(1, 1, 1, 1);
			Utils.renderMultiColumnText(0.01, 0.9, getCorrectTextSize(0.013), strs, 0.008, {RenderText.ALIGN_LEFT, RenderText.ALIGN_LEFT});
		end;
	end;
end;

function GameExtension:draw()
	GameExtension.callFunc(self, "draw", {});
end;



function GameExtension:saveSavegame(superFunc, ...)
	if superFunc ~= nil then
		superFunc(self, ...);
	end;
	
	if g_currentMission:getIsServer() then
		GameExtension.saveFile(GameExtension.serverFilename, true);
	end;
	
	-- Not needed as we auto save client settings
	-- if g_currentMission:getIsClient() and g_dedicatedServerInfo == nil then
		-- GameExtension.saveFile(GameExtension.clientFilename, false);
	-- end;
end;
GameExtension.addClassOverride("OVERRIDE_SAVE_GAME", g_careerScreen, "saveSavegame", GameExtension.saveSavegame);


-- We must finish loading the file before we remove the class
if GameExtension.modName ~= "FS17_GameExtension" then
	log("ERROR", "FS17_GameExtension have been renamed to " .. GameExtension.modName .. " and will not function properly, deactivating " .. GameExtension.modName .. " fully.");
	
	GameExtension = nil;
end;