--
-- SynchSettingsEvent
--
-- @author:    	Xentro (Marcus@Xentro.se)
-- @website:	www.Xentro.se
-- @history:	v1.0 - 2016-11-04 - Initial implementation
-- 


SynchSettingsEvent = {};
SynchSettingsEvent.ServerModules = {};

SynchObjectListEvent_mt = Class(SynchSettingsEvent, Event);

InitEventClass(SynchSettingsEvent, "SynchSettingsEvent");

function SynchSettingsEvent:emptyNew()
    local self = Event:new(SynchObjectListEvent_mt);
	
    self.className = "SynchSettingsEvent";
	
    return self;
end;

function SynchSettingsEvent:new()
	local self = SynchSettingsEvent:emptyNew();
	
    return self;
end;

function SynchSettingsEvent:readStream(streamId, connection)
	if connection:getIsServer() then -- client
		log("DEBUG", "Starting to recive update from server");
		
		local numModules = streamReadInt16(streamId);
		log("DEBUG", "We got " .. numModules .. " modules to update.");
		
		for i = 1, numModules do
			local moduleName = streamReadString(streamId);
			local numSettings = streamReadInt16(streamId);
			local isActive = streamReadBool(streamId);
			
			-- log("DEBUG", "Reading: " .. moduleName .. ", state: " .. tostring(isActive));
			GameExtension.setSetting(moduleName, nil, isActive, true);
			
			if not GameExtension.getBlackListItem(moduleName) then
				for k = 1, numSettings do
					local name = streamReadString(streamId);
					local formatType = streamReadString(streamId);
					local value; 
					
					if formatType == "string" then
						value = streamReadString(streamId);
					elseif formatType == "float" then
						value = streamReadFloat32(streamId);
					elseif formatType == "int" then
						value = streamReadInt16(streamId);
					else
						value = streamReadBool(streamId);
					end;
					
					-- log("DEBUG", "  Reading: " .. name .. ", value: " .. tostring(value) .. ", type: " .. tostring(formatType));
					GameExtension.setSetting(moduleName, name, value, true);
				end;
			end;
		end;
	end;
end;

function SynchSettingsEvent:writeStream(streamId, connection)
	if not connection:getIsServer() then -- server
		log("DEBUG", "Starting to send update to clients");
		log("DEBUG", "We are sending " .. #SynchSettingsEvent.ServerModules .. " modules updates to client.");
		
		streamWriteInt16(streamId, #SynchSettingsEvent.ServerModules);
		
		for _, v in pairs(SynchSettingsEvent.ServerModules) do
			streamWriteString(streamId, v.moduleName);
			streamWriteInt16(streamId, v.numSettings);
			
			if not GameExtension.getBlackListItem(v.moduleName) then
				streamWriteBool(streamId, GameExtension.getSetting(v.moduleName));
				local settings = GameExtension.getSetting(v.moduleName, nil, true);
				
				-- log("DEBUG", "Writing: " .. v.moduleName .. ", state: " .. tostring(GameExtension.getSetting(v.moduleName)));
				
				for name, v in pairs(settings) do
					if not GameExtension.getBlackListItem(name) and v.allowEvent then
						-- log("DEBUG", "  Writing: " .. name .. ", value: " .. tostring(v.value) .. ", type: " .. tostring(v.type));
						
						local formatType = v.type;
						streamWriteString(streamId, name);
						streamWriteString(streamId, formatType);
						
						if formatType == "string" then
							streamWriteString(streamId, v.value);
						elseif formatType == "float" then
							streamWriteFloat32(streamId, v.value);
						elseif formatType == "int" then
							streamWriteInt16(streamId, v.value);
						else
							streamWriteBool(streamId, v.value);
						end;
					end;
				end;
			else
				streamWriteBool(streamId, false);
			end;
		end;
	end;
end;

function SynchSettingsEvent:run(connection)
end;

function SynchSettingsEvent.updateServerModules()
	-- log("DEBUG", "Creating list of modules to send", true);
	
	for moduleName, setting in pairs(GameExtension.modules) do
		if not GameExtension.getBlackListItem(moduleName) then
			local num = 0;
			
			for name, v in pairs(setting) do
				if not GameExtension.getBlackListItem(name) and v.allowEvent then
					num = num + 1;
				end;
			end;
			
			-- log("DEBUG", "Module " .. moduleName .. " has " .. num .. " settings to be sent", true);
			table.insert(SynchSettingsEvent.ServerModules, {moduleName = moduleName, numSettings = num});
		end;
	end;
end;



if not Utils.joinFix20150920 then
	Utils.joinFix20150920 = true;
	Utils.playerJoinEventsToSend = {};
	
	local oldSendObjects = Server.sendObjects;
	function Server:sendObjects(connection, x, y, z, viewDistanceCoeff)
		-- log("DEBUG", "Synching all settings with playes! (playerJoinEventsToSend)", true);
		
		for _, class in ipairs(Utils.playerJoinEventsToSend) do
			-- log("DEBUG", "Calling event class!", true);
			
			connection:sendEvent(class:new());
		end;
		
		oldSendObjects(self, connection, x, y, z, viewDistanceCoeff);
	end;
end;



if GameExtension ~= nil then
	SynchSettingsEvent.updateServerModules();
	table.insert(Utils.playerJoinEventsToSend, SynchSettingsEvent); -- Add to synch table
end;