--
-- AI
--
-- @author:    	Xentro (Marcus@Xentro.se)
-- @website:	www.Xentro.se
-- @history:	v1.0 - 2016-11-19 - Initial implementation
-- 

AI = {};

function AI:loadMap()
	GameExtension.addSpecialization("AI_SPEC", AI_SPEC);
	
	for i, name in pairs({"aiVehicleReasonShiftEnded", "aiVehicleReasonFired"}) do
		GameExtension.makeTextGlobal(name);
		table.insert(AIVehicle.REASON_TEXT_MAPPING, name);
		
		if i == 1 then
			AIVehicle.STOP_REASON_SHIFT_ENDED = #AIVehicle.REASON_TEXT_MAPPING;
		elseif i == 2 then
			AIVehicle.STOP_REASON_FIRED = #AIVehicle.REASON_TEXT_MAPPING;
		end;
	end;
	
	AIVehicle.NUM_BITS_REASONS = AIVehicle.NUM_BITS_REASONS + 1; -- 4 = 16 bits, 8 texts by default + 2 = 10 bits needed
	
	self.turnOffTimer = {updates = 7, current = 0};
	g_currentMission.GE_stopOneAI = false;
	
	g_currentMission.GE_isAIOvertime = false;
end;

function AI:delete()
end;

function AI:updateTick(dt)
	if GameExtension.getSetting("AI", "AI_ACTIVE") then
		-- Reduce fetching
		g_currentMission.GE_isAIOvertime = g_currentMission.environment.currentHour >= GameExtension.getSetting("AI", "AI_WORK_HOUR_END") or g_currentMission.environment.currentHour < GameExtension.getSetting("AI", "AI_WORK_HOUR_START");
		
		if g_currentMission:getIsServer() then
			if AIVehicle.numHirablesHired > g_currentMission.maxNumHirables then
				if not g_currentMission.GE_stopOneAI then
					self.turnOffTimer.current = self.turnOffTimer.current + 1;
					
					if self.turnOffTimer.current > self.turnOffTimer.updates then
						g_currentMission.GE_stopOneAI = true;
						self.turnOffTimer.current = 0;
					end;
				end;
			else
				g_currentMission.GE_stopOneAI = false;
				self.turnOffTimer.current = 0;
			end;
		end;
	else
		g_currentMission.GE_isAIOvertime = false;
	end;
end;

function AI:setMaxNumHireables(num)
	g_currentMission.maxNumHirables = num;
end;

function AI:setAIPrice(value)
	-- we could update the AI price now...
end;



-- Fix, we tell the AI to turn off engine when its done
function AI.onStopAiVehicle(self, oldFunc)
	oldFunc(self);
	
	if not self.isEntered then
		self:stopMotor(true);
		self:onLeave(); -- turn everything off!
		
		if self.startTurnOffTimer == nil then
			self.startTurnOffTimer = 750;
			self.forceIsActive = true;
		end;
	end;
end;

-- Fix, we arent smart enough to start engine
function AI.onStartAiVehicle(self, oldFunc)
	if not self.isEntered and not self.isMotorStarted then
		self:startMotor(true);
		-- self:onEnter();
	end;
	
	oldFunc(self);
end;

function AI.canStartAIVehicle(self, oldFunc)
	local state = oldFunc(self);
	
	if g_currentMission.GE_isAIOvertime then
		-- We might be able to use g_currentMission.disableAIVehicle instead but lets do this for now
		return false;
	end;
	
	return state;
end;

-- AIVehicle.onStopAiVehicle = Utils.overwrittenFunction(AIVehicle.onStopAiVehicle, AI.onStopAiVehicle);
-- AIVehicle.onStartAiVehicle = Utils.overwrittenFunction(AIVehicle.onStartAiVehicle, AI.onStartAiVehicle);
-- AIVehicle.canStartAIVehicle = Utils.overwrittenFunction(AIVehicle.canStartAIVehicle, AI.canStartAIVehicle);
GameExtension.addClassOverride("OVERRIDE_AI_ON_STOP", AIVehicle, "onStopAiVehicle", AI.onStopAiVehicle);
GameExtension.addClassOverride("OVERRIDE_AI_ON_START", AIVehicle, "onStartAiVehicle", AI.onStartAiVehicle);
GameExtension.addClassOverride("OVERRIDE_AI_CAN_START", AIVehicle, "canStartAIVehicle", AI.canStartAIVehicle);




AI_SPEC = {};

function AI_SPEC:postLoad(saveGame)
	self.gameExtension.AI = {};
	self.gameExtension.AI.isActive = SpecializationUtil.hasSpecialization(AIVehicle, self.specializations);
	self.gameExtension.AI.oldPrice = self.pricePerMS;
	self.gameExtension.AI.last = self.gameExtension.AI.oldPrice;
	self.gameExtension.AI.active = false;
	self.gameExtension.AI.resumeWork = false;
	self.gameExtension.AI.outsideDisturbance = false;
	self.gameExtension.AI.resumeHelperIndex = nil;
	
	-- addMessage("numHirablesHired", AIVehicle);
end;

function AI_SPEC:updateTick(dt)
	if not self.gameExtension.AI.isActive then
		return;
	end;
	
	-- Give time to stop particles and such
	if self.startTurnOffTimer ~= nil then
		self.startTurnOffTimer = self.startTurnOffTimer - dt;
		if self.startTurnOffTimer < 0 then
			self.startTurnOffTimer = nil;
			self.forceIsActive = false;
		end;
	end;
	
	if self.isServer then
		if GameExtension.getSetting("AI", "AI_ACTIVE") then
			if (self:getIsActive() or self.gameExtension.AI.resumeWork) then
				self.gameExtension.AI.active = true;
			
				if self.aiIsStarted then
					if not self.gameExtension.AI.outsideDisturbance then
						if self.pricePerMS ~= self.gameExtension.AI.last then
							self.gameExtension.AI.outsideDisturbance = true;
							log("WARNING", "AI pricePerMS have been changed by another mod, AI Rate Extra and AI Rate Normal settings will not function properly from now!");
						end;
						
						-- We could pass this trough an function but this only change on server side.. leave as is for now.
						if not self.gameExtension.AI.outsideDisturbance then 
							local price = GameExtension.getSetting("AI", "AI_RATE_NORMAL_MULTIPLIER");
							
							if g_currentMission.GE_isAIOvertime then
								price = GameExtension.getSetting("AI", "AI_RATE_EXTRA_MULTIPLIER");
							end;
							
							price = self.gameExtension.AI.oldPrice * price;
							
							if price ~= self.gameExtension.AI.last then
								self.pricePerMS = price;
								self.gameExtension.AI.last = price;
							end;
						end;
					end;
				end;
			
				if g_currentMission.GE_isAIOvertime then
					if self.aiIsStarted then
						if GameExtension.getSetting("AI", "AI_WORKDAY_ENDED") then
							self.gameExtension.AI.resumeWork = true;
							self.gameExtension.AI.resumeHelperIndex = self.currentHelper.index;
							
							self:stopAIVehicle(AIVehicle.STOP_REASON_SHIFT_ENDED);
						end;
					end;
				else
					if not self.aiIsStarted and self.gameExtension.AI.resumeWork then
						self:startAIVehicle(self.gameExtension.AI.resumeHelperIndex);
						
						self.gameExtension.AI.resumeWork = false;
						self.gameExtension.AI.resumeHelperIndex = nil;
					end;
				end;
				
				if g_currentMission.GE_stopOneAI then
					g_currentMission.GE_stopOneAI = false;
					self:stopAIVehicle(AIVehicle.STOP_REASON_FIRED);
				end;
			end;
		elseif self.gameExtension.AI.active then
			self.gameExtension.AI.active = false;
			self.gameExtension.AI.resumeWork = false;
			self.gameExtension.AI.resumeHelperIndex = nil;
			self.pricePerMS = self.gameExtension.AI.oldPrice;
		end;
	end;
end;