--
-- CRUISE_CONTROL
--
-- @author:    	Xentro (Marcus@Xentro.se)
-- @website:	www.Xentro.se
-- @history:	v1.0 - 2016-10-29 - Initial implementation
-- 

CRUISE_CONTROL = {};

function CRUISE_CONTROL:load(savegame)
	self.gameExtension.cruiseControl = SpecializationUtil.hasSpecialization(Drivable, self.specializations);
end;

function CRUISE_CONTROL:update(dt)
	if not self.gameExtension.cruiseControl then
		return;
	end;
	
	if self:getIsActive() then
		if GameExtension.getSetting("CRUISE_CONTROL", "CRUISE_ACTIVE") then
			if self:getIsActiveForInput(false, false) then
				if InputBinding.hasEvent(InputBinding.X_CRUISE_ACTIVATE) then
					if self.cruiseControl.state == Drivable.CRUISECONTROL_STATE_OFF then
						self:setCruiseControlState(Drivable.CRUISECONTROL_STATE_ACTIVE);
					else
						self:setCruiseControlState(Drivable.CRUISECONTROL_STATE_OFF);
					end;
					
				elseif InputBinding.isPressed(InputBinding.X_CRUISE_ACTIVATE) then
					self.cruiseControl.customTopSpeedTimer = self.cruiseControl.customTopSpeedTimer - dt;
					
					if self.cruiseControl.customTopSpeedTimer < 0 then
						self:setCruiseControlState(Drivable.CRUISECONTROL_STATE_FULL);
					end;
				else
					self.cruiseControl.customTopSpeedTimer = 500;
				end;
				
				if InputBinding.isPressed(InputBinding.X_CRUISE_DOWN) then
					CRUISE_CONTROL.updateCruiseSpeed(self, dt, -1);
				elseif InputBinding.isPressed(InputBinding.X_CRUISE_UP) then
					CRUISE_CONTROL.updateCruiseSpeed(self, dt, 1);
				end;
			end;
		end;
	end;
end;

function CRUISE_CONTROL:updateCruiseSpeed(dt, dir)
	self.cruiseControl.changeCurrentDelay = self.cruiseControl.changeCurrentDelay + dt * (self.cruiseControl.changeMultiplier * dir);
	self.cruiseControl.changeMultiplier = Utils.clamp(self.cruiseControl.changeMultiplier + dt * (GameExtension.getSetting("CRUISE_CONTROL", "CRUISE_SCROOL_SPEED") * dir), 0, 10);
	
	self.cruiseControl.wasSpeedChanged = true; -- no purpose, dont hurt to keep it.
	self:setCruiseControlMaxSpeed(self.cruiseControl.speed + dir);
	self.cruiseControl.changeCurrentDelay = self.cruiseControl.changeDelay;
	-- speed will be updated for MP on the next frame by default script.
end;