--
-- DYNAMIC_HOSE
--
-- @author:    	Xentro (Marcus@Xentro.se)
-- @website:	www.Xentro.se
-- @history:	v1.0 - 2016-11-26 - Initial implementation
-- 

DYNAMIC_HOSE = {};

function DYNAMIC_HOSE:loadMap()
	self.DynamicHose 							  = {};
	self.DynamicHose.refVehicles 	  			  = {};
	self.DynamicHose.hoseVehicles 				  = {};
	self.DynamicHose.alwaysFunction 			  = false;
	self.DynamicHose.missingOneHoseFunctionAnyway = true; -- All hoses of that type must be attached for that type to function.
	
	-- We need these for the vehicles
	self.addRefVehicle = DYNAMIC_HOSE.addRefVehicle;
	self.removeRefVehicle = DYNAMIC_HOSE.removeRefVehicle;
	self.addHoseVehicle = DYNAMIC_HOSE.addHoseVehicle;
	self.removeHoseVehicle = DYNAMIC_HOSE.removeHoseVehicle;
	
	self.getHoseSetting = DYNAMIC_HOSE.getHoseSetting;
	
	-- Supported hoses
	g_currentMission.dynamicHoseTypes 				= {};
	g_currentMission.dynamicHoseTypes["hydraulic"] 	= true;
	g_currentMission.dynamicHoseTypes["electric"] 	= true;
	g_currentMission.dynamicHoseTypes["air"] 		= true;
	
	-- Use to override the attaching/detaching of hoses, Manual Attaching will use this.
	-- g_currentMission.dynamicHoseIsManual = true;
	
	g_currentMission.dynamicHose = self;
end;

function DYNAMIC_HOSE:deleteMap()
	g_currentMission.dynamicHose = nil;
	g_currentMission.dynamicHoseTypes = nil;
end;

function DYNAMIC_HOSE:updateTick()
end;



function DYNAMIC_HOSE:getHoseSetting(name)
	if self.DynamicHose[name] ~= nil then
		return self.DynamicHose[name];
	end;
	
	return true;
end;



function DYNAMIC_HOSE:addRefVehicle(vehicle, version)
	-- table.insert(self.DynamicHose.refVehicles, vehicle);
end;

function DYNAMIC_HOSE:addHoseVehicle(vehicle, version)
	-- local modName = Utils.getModNameAndBaseDirectory(vehicle.configFileName);
	-- local tableName = "DynamicHose";
	-- local mod = _G[modName];
	
	-- if mod ~= nil and mod[tableName] ~= nil then
		-- mod[tableName].load = Utils.overwrittenFunction(mod[tableName].load, DYNAMIC_HOSE.test);
	-- end;
	
	table.insert(self.DynamicHose.hoseVehicles, vehicle);
end;



function DYNAMIC_HOSE:removeRefVehicle(vehicle)
	-- DYNAMIC_HOSE.removeVehicle(self, vehicle, self.DynamicHose.refVehicles);
end;

function DYNAMIC_HOSE:removeHoseVehicle(vehicle)
	DYNAMIC_HOSE.removeVehicle(self, vehicle, self.DynamicHose.hoseVehicles);
end;

function DYNAMIC_HOSE:removeVehicle(vehicle, t)
	for i, v in ipairs(t) do
		if v == vehicle then
			table.remove(t, i);
			break;
		end;
	end;
end;

function DYNAMIC_HOSE:setFunctionAnyway(state)
	if state == 2 then
		-- We arent allowed to change light state after missingOneHoseFunctionAnyway is true
		DYNAMIC_HOSE.vehicleDifficultyUpdate(self, true);
	end;
	
	-- 0 - Auto Attach
	-- 1 - One hose of the type must be attached for the type to function
	-- 2 - All hoses of the type must be attached for the type to function
	
	self.DynamicHose.alwaysFunction					= state == 0; 	-- true - 0
	self.DynamicHose.missingOneHoseFunctionAnyway   = state <= 1;	-- true - 0, 1
	
	if g_currentMission.callbackManualAttaching ~= nil then			-- Manual Attach mod is activated
		g_currentMission.dynamicHoseIsManual 		= state >= 1;	-- true - 1, 2
	end;
	
	if state ~= 2 then
		-- synch settings after missingOneHoseFunctionAnyway is changed from false
		DYNAMIC_HOSE.vehicleDifficultyUpdate(self, false);
	end;
end;

function DYNAMIC_HOSE:vehicleDifficultyUpdate(isHardLevel)
	for i, v in ipairs(self.DynamicHose.hoseVehicles) do
		if v.gameExtensionDifficultyUpdate ~= nil then
			v:gameExtensionDifficultyUpdate(isHardLevel);
		end;
	end;
end;