--
-- PLAYER
--
-- @author:    	Xentro (Marcus@Xentro.se)
-- @website:	www.Xentro.se
-- @history:	v1.0 - 2016-11-19 - Initial implementation
-- 

PLAYER = {};

function PLAYER:loadMap()
	self.playerStuff = {};
	self.playerStuff.mass 		= {old = Player.MAX_PICKABLE_OBJECT_MASS};		-- Default: 0.2 Mass
	-- self.playerStuff.distance 	= {old = Player.MAX_PICKABLE_OBJECT_DISTANCE};	-- Default: 3 Meters
	
	if g_currentMission:getIsClient() then
		self.playerStuff.crosshair = {last = true};
		
		local uiScale = GameExtension.getSetting("PLAYER", "FARMERS_TOUCH_HUD_SCALE") * g_gameSettings:getValue("uiScale");
		
		self.farmersThouchOverlays = {};
		self.showGrowthStateAt = {};
		self.showGrowthStateAt[1] = {start = {0, 0, 0}, width = {0, 0, 0}, height = {0, 0, 0}};
		self.multiplyHudsOffset = {getNormalizedScreenValues(0, 50 * uiScale)};
		
		PLAYER.createFarmerThouchOverlay(self, 1);
		
		local textPosition1 = {getNormalizedScreenValues(78 * uiScale, 17 * uiScale)};
		local textPosition2 = {getNormalizedScreenValues(227 * uiScale, 0)};
		local textPosition3 = {getNormalizedScreenValues(330 * uiScale, 0)};
		local textPosition4 = {getNormalizedScreenValues(435 * uiScale, 0)};
		self.farmersThouchTextPosition = {};
		self.farmersThouchTextPosition[1] = {x = textPosition1[1], y = textPosition1[2], textsize = getCorrectTextSize(0.02 * uiScale)};
		self.farmersThouchTextPosition[2] = {x = textPosition2[1], y = textPosition1[2], textsize = getCorrectTextSize(0.02 * uiScale)};
		self.farmersThouchTextPosition[3] = {x = textPosition3[1], y = textPosition1[2], textsize = getCorrectTextSize(0.02 * uiScale)};
		self.farmersThouchTextPosition[4] = {x = textPosition4[1], y = textPosition1[2], textsize = getCorrectTextSize(0.02 * uiScale)};
		
		self.farmersThouchKeyTimer = {current = 0, max = 500, called = false};
	end;
end;

function PLAYER:deleteMap()
	for i, v in ipairs(self.farmersThouchOverlays) do
		v[1]:delete();
		v[2]:delete();
	end;
end;

function PLAYER:update(dt)
	if g_currentMission:getIsClient() then
		local p = g_currentMission.player;
		
		if g_currentMission.controlPlayer and p ~= nil and p.rootNode ~= nil then
			if GameExtension.getSetting("PLAYER", "FARMERS_TOUCH_ACTIVE") then
				local meter = 0.05; -- 25 cm2
				local start = {localToWorld(p.rootNode,  meter, 0.3, -meter)};
				local width = {localToWorld(p.rootNode, -meter, 0.3, -meter)};
				local height = {localToWorld(p.rootNode, meter, 0.3,  meter)};
				
				self.showGrowthStateAt[1].start = start;
				self.showGrowthStateAt[1].width = width;
				self.showGrowthStateAt[1].height = height;
				
				if self:getIsActiveForInput() then
					if InputBinding.isPressed(InputBinding.X_FARMERS_TOUCH) then
						if self.farmersThouchKeyTimer.current <= 0 then
							if not self.farmersThouchKeyTimer.called then
								PLAYER.emptyFarmersTouch(self);
								self.farmersThouchKeyTimer.called = true;
							end;
						else
							self.farmersThouchKeyTimer.current = self.farmersThouchKeyTimer.current - dt;
						end;
					else
						if self.farmersThouchKeyTimer.current > 0 and self.farmersThouchKeyTimer.current < self.farmersThouchKeyTimer.max then
							if #self.showGrowthStateAt < GameExtension.getSetting("PLAYER", "FARMERS_TOUCH_NUM_HUDS") then
								table.insert(self.showGrowthStateAt, {start = start, width = width, height = height});
								PLAYER.createFarmerThouchOverlay(self, #self.showGrowthStateAt);
							else
								-- TODO: Show max number warning
							end;
						end;
						
						self.farmersThouchKeyTimer.current = self.farmersThouchKeyTimer.max;
						self.farmersThouchKeyTimer.called = false;
					end;
				end;
			end;
		end;
	end;
end;

function PLAYER.playerReadUpdate(self, oldFunc, streamId, timestamp, connection)
	oldFunc(self, streamId, timestamp, connection);
	
	if connection:getIsServer() then
		-- read from server
		
		if self.isObjectInRange then
			if streamReadBool(streamId) then
				self.farmersTouchObject = readNetworkNodeObject(streamId);
			else
				self.farmersTouchObject = nil;
			end;
		end;
	end;
end;

function PLAYER.playerWriteUpdate(self, oldFunc, streamId, connection, dirtyMask)
	oldFunc(self, streamId, connection, dirtyMask);
	
	if not connection:getIsServer() then
		-- send to client
		
		if self.isObjectInRange then
			streamWriteBool(streamId, self.farmersTouchObject ~= nil);
			
			if self.farmersTouchObject ~= nil then
				writeNetworkNodeObject(streamId, self.farmersTouchObject);
			end;
		end;
	end;
end;
GameExtension.addClassOverride("OVERRIDE_PLAYER_READ_UPDATE", Player, "readUpdateStream", PLAYER.playerReadUpdate);
GameExtension.addClassOverride("OVERRIDE_PLAYER_WRITE_UPDATE", Player, "writeUpdateStream", PLAYER.playerWriteUpdate);


function PLAYER.playerUpdate(self, oldFunc, ...)
	oldFunc(self, ...);
	
	if self.isControlled and self.isServer then
		if self.isObjectInRange then
			local object = g_currentMission:getNodeObject(self.lastFoundObject);
			
			if object ~= nil then
				self.farmersTouchObject = object;
			end;
		else
			self.farmersTouchObject = nil;
		end;
	end;
end;
GameExtension.addClassOverride("OVERRIDE_PLAYER_UPDATE", Player, "update", PLAYER.playerUpdate);


function PLAYER:updateTick(dt)
	if g_currentMission:getIsClient() then
		if g_currentMission.controlPlayer then
			local player = g_currentMission.player;
			
			if self.firstTimeRun then
				-- Player Crosshair
				if player ~= nil then
					if (GameExtension.getSetting("PLAYER", "CROSSHAIR") or player.isObjectInRange) then
						if g_currentMission.showHudEnv then
							PLAYER.setCrosshairState(self, true, player);
						end;
					else
						PLAYER.setCrosshairState(self, false, player);
					end;
				end;
			end;
			
			-- Farmers Thouch
			if GameExtension.getSetting("PLAYER", "FARMERS_TOUCH_ACTIVE") then
				for i, v in ipairs(self.showGrowthStateAt) do
					local density, numPixels, growthState, totalPixels, found;
					local avoid = {grass = true, dryGrass = true};
					
					if i == 1 or v.index == nil then
						for k, desc in pairs(FruitUtil.fruitIndexToDesc) do
							if not avoid[desc.name] then
								density, numPixels, growthState, totalPixels = GameExtension.getFruitArea(desc.index, v.start[1], v.start[3], v.width[1], v.width[3], v.height[1], v.height[3]);
								
								-- showMessage(desc.name, "density: " .. density .. " numPixels: " .. numPixels .. " totalPixels: " .. totalPixels .. " stage: " .. tostring(growthState));
								
								if numPixels > 0 then
									found = desc;
									
									if i >= 2 then		-- Avoid the player
										v.index = k; 	-- Avoid necessary loops 
									end;
									
									break;
								end;
							end;
						end;
						
						-- Override field data with pallet/bale data
						if i == 1 and player ~= nil then
							-- showMessage("lastFoundObject", player);
							-- showMessage("pickedUpObject", player);
							-- showMessage("farmersTouchObject", player);
							
							
							if player.isObjectInRange and player.farmersTouchObject ~= nil then
								local object = player.farmersTouchObject;
								
								if object ~= nil then
									if object:isa(FillablePallet) or object:isa(Bale) then
										local fillDesc = FillUtil.fillTypeIndexToDesc[object.fillType];
										
										if fillDesc ~= nil then
											if v.foundObject == nil then
												v.foundObject = {};
											end;
											
											found = nil;
											
											v.foundObject.name = fillDesc.nameI18N;
											
											if object:isa(Bale) then
												v.foundObject.status = string.format(g_i18n:getText("GROWTH_OBJECT_WEIGHT"), player.lastFoundObjectMass);
											else
												v.foundObject.status = math.floor((object.fillLevel / object.capacity) * 100) .. "%";
											end;
											
											v.foundObject.realLevel = math.floor(object.fillLevel);
										end;
									end;
								end;
							else
								v.foundObject = nil;
							end;
						end;
					else
						density, numPixels, growthState, totalPixels = GameExtension.getFruitArea(v.index, v.start[1], v.start[3], v.width[1], v.width[3], v.height[1], v.height[3]);
						
						-- showMessage("-" .. FruitUtil.fruitIndexToDesc[v.index].name, "density: " .. density .. " numPixels: " .. numPixels .. " totalPixels: " .. totalPixels .. " stage: " .. tostring(growthState));
						
						if numPixels > 0 then
							found = FruitUtil.fruitIndexToDesc[v.index];
						end;
					end;
					
					if found ~= nil then
						local fillDesc = FillUtil.fillTypeNameToDesc[found.name];
						if v.foundField == nil then
							v.foundField = {};
						end;
						
						if fillDesc ~= nil then
							v.foundField.name = fillDesc.nameI18N;
						else
							v.foundField.name = found.name;
						end;
						
						local minGrowthOffset = found.minHarvestingGrowthState + 1;
						local maxGrowthOffset = found.maxHarvestingGrowthState + 1;
						
						
						if found.maxPreparingGrowthState > 0 then
							minGrowthOffset = found.minPreparingGrowthState + 1;
							maxGrowthOffset = found.maxPreparingGrowthState + 1;
						end;
						
						v.foundField.status  	= math.floor((growthState / maxGrowthOffset) * 100);
						
						if growthState == 1 then
							v.state = PLAYER.FT_SEEDED;
						elseif found.maxPreparingGrowthState > 0 and maxGrowthOffset - minGrowthOffset >= 2 and growthState == minGrowthOffset then
							v.state = PLAYER.FT_PRE_PREPARE;
						elseif found.maxPreparingGrowthState > 0 and growthState >= minGrowthOffset and growthState <= maxGrowthOffset then
							v.state = PLAYER.FT_PREPARE;
						elseif growthState < minGrowthOffset then
							v.state = PLAYER.FT_GROWING;
						elseif maxGrowthOffset - minGrowthOffset >= 2 and growthState == minGrowthOffset then
							v.state = PLAYER.FT_PRE_HARVEST;
						elseif growthState >= minGrowthOffset and growthState <= maxGrowthOffset then
							v.state = PLAYER.FT_HARVEST;
						elseif growthState == maxGrowthOffset + 1 then
							v.state = PLAYER.FT_DEAD;
						elseif growthState == maxGrowthOffset + 2 then
							v.state = PLAYER.FT_HARVESTED;
						else
							v.state = PLAYER.FT_PREPARED;
						end;
					else
						v.foundField = nil;
						
						-- Nothing is growing at location, clean up.
						if i >= 2 and self.farmersThouchOverlays[i] ~= nil then
							self.showGrowthStateAt[i] = nil;
							self.farmersThouchOverlays[i][1]:delete();
							self.farmersThouchOverlays[i][2]:delete();
							self.farmersThouchOverlays[i] = nil;
						end;
					end;
				end;
			end;
		else
			-- Do we reset the markers?
		end;
	end;
end;

-- Density value from channel in comment
PLAYER.FT_SEEDED 		= 0; -- 1
PLAYER.FT_GROWING 		= 1; -- 2 - 4
PLAYER.FT_PRE_PREPARE 	= 2; -- 5 - 6
PLAYER.FT_PREPARE 		= 3; -- 7
PLAYER.FT_PRE_HARVEST 	= 4; -- 5 - 6
PLAYER.FT_HARVEST 		= 5; -- 7
PLAYER.FT_DEAD 			= 6; -- 8
PLAYER.FT_HARVESTED 	= 7; -- 9
PLAYER.FT_PREPARED 		= 8; -- 10

--[[
Growth states from SoilMod by Decker_MMIV
   Density value (from channels/bits)
   |  RegisterFruit value (for RegisterFruit)
   |  |
   0  -  nothing
   1  0  growth-1 (just seeded)
   2  1  growth-2
   3  2  growth-3
   4  3  growth-4
   5  4  harvest-1 / prepare-1
   6  5  harvest-2 / prepare-2
   7  6  harvest-3 / prepare-3
   8  7  withered
   9  8  cutted
  10  9  harvest (defoliaged)
  11 10  <unused>
  12 11  <unused>
  13 12  <unused>
  14 13  <unused>
  15 14  <unused>
--]]

function PLAYER:draw()
	if g_currentMission:getIsClient() then
		if g_currentMission.showHudEnv and g_currentMission.controlPlayer then
			if GameExtension.getSetting("PLAYER", "FARMERS_TOUCH_ACTIVE") then
				for i, v in ipairs(self.showGrowthStateAt) do
					local overlay = self.farmersThouchOverlays[i];
					
					if overlay ~= nil and (v.foundField ~= nil or v.foundObject ~= nil) then
						local position = {overlay[1].x, overlay[1].y};
						local txt, txtColor, status = "";
						
						setTextColor(1, 1, 1, 1);
						setTextAlignment(RenderText.ALIGN_CENTER); 
						
						overlay[1]:render();
						overlay[2]:render();
						setTextBold(true);
						
						if v.foundField ~= nil then
							renderText(position[1] + self.farmersThouchTextPosition[1].x, position[2] + self.farmersThouchTextPosition[1].y, self.farmersThouchTextPosition[1].textsize, g_i18n:getText("GROWTH_STATUS"));
							setTextBold(false);
							renderText(position[1] + self.farmersThouchTextPosition[2].x, position[2] + self.farmersThouchTextPosition[2].y, self.farmersThouchTextPosition[2].textsize, v.foundField.name);
							
							if v.state == PLAYER.FT_SEEDED then
								txt = g_i18n:getText("GROWTH_STATE_SEEDED");
							elseif v.state == PLAYER.FT_GROWING then
								txt = g_i18n:getText("GROWTH_STATE_GROWING");
								status = v.foundField.status;
							elseif v.state == FT_PRE_PREPARE or v.state == PLAYER.FT_PREPARE then
								txt = g_i18n:getText("GROWTH_STATE_PREPARE");
								status = v.foundField.status;
								
								if v.state == FT_PRE_PREPARE then
									txtColor = GameExtension.colorCodes["TEXT_YELLOW"];
								else
									txtColor = GameExtension.colorCodes["TEXT_GREEN"];
								end;
							elseif v.state == PLAYER.FT_PRE_HARVEST or v.state == PLAYER.FT_HARVEST or v.state == PLAYER.FT_PREPARED then
								txt = g_i18n:getText("GROWTH_STATE_READY");
								
								if v.state ~= PLAYER.FT_PREPARED then
									status = v.foundField.status;
								end;
								
								if v.state == PLAYER.FT_PRE_HARVEST then
									txtColor = GameExtension.colorCodes["TEXT_YELLOW"];
								else
									txtColor = GameExtension.colorCodes["TEXT_GREEN"];
								end;
							elseif v.state == PLAYER.FT_DEAD then
								txt = g_i18n:getText("GROWTH_STATE_DEAD");
								txtColor = GameExtension.colorCodes["TEXT_RED"];
							-- elseif v.state == PLAYER.FT_PREPARED then
								-- We dont need anything custom for this, use ready.
							elseif v.state == PLAYER.FT_HARVESTED then
								txt = g_i18n:getText("GROWTH_STATE_HARVESTED");
							end;
							
							if txt == "" then
								txt = "missing state: " .. v.state;
								status = v.foundField.status;
							end;
							
							if status ~= nil then
								renderText(position[1] + self.farmersThouchTextPosition[3].x, position[2] + self.farmersThouchTextPosition[3].y, self.farmersThouchTextPosition[3].textsize, status .. "%");
							end;
							if txtColor ~= nil then
								setTextColor(unpack(txtColor));
							end;
							
							renderText(position[1] + self.farmersThouchTextPosition[4].x, position[2] + self.farmersThouchTextPosition[4].y, self.farmersThouchTextPosition[4].textsize, txt);
						
						elseif v.foundObject ~= nil then
							renderText(position[1] + self.farmersThouchTextPosition[1].x, position[2] + self.farmersThouchTextPosition[1].y, self.farmersThouchTextPosition[1].textsize, g_i18n:getText("GROWTH_OBJECT"));
							setTextBold(false);
							renderText(position[1] + self.farmersThouchTextPosition[2].x, position[2] + self.farmersThouchTextPosition[2].y, self.farmersThouchTextPosition[2].textsize, v.foundObject.name);
							renderText(position[1] + self.farmersThouchTextPosition[3].x, position[2] + self.farmersThouchTextPosition[3].y, self.farmersThouchTextPosition[3].textsize, v.foundObject.status);
							renderText(position[1] + self.farmersThouchTextPosition[4].x, position[2] + self.farmersThouchTextPosition[4].y, self.farmersThouchTextPosition[4].textsize, v.foundObject.realLevel .. "L");
						end;
						
						setTextColor(1, 1, 1, 1);
						setTextAlignment(RenderText.ALIGN_LEFT); 
					end;
				end;
			end;
		end;
	end;
end;

function PLAYER:setCrosshairState(state, player)
	-- if player.pickedUpObjectOverlay.visible ~= state then
	if self.playerStuff.crosshair.last ~= state then
		self.playerStuff.crosshair.last = state;
		player.pickedUpObjectOverlay:setIsVisible(state);
	end;
end;

function PLAYER:updateCrosshair(state)
	if not g_currentMission.showHudEnv and self.crosshairWasActivated ~= nil then
	-- if not state and GameExtension.getSetting("SCREENSHOT") then
		self.crosshairWasActivated = nil;
	end;
end;

function PLAYER:setPlayerStrenth(strength)
	-- log("DEBUG", "Player Strenght: " .. strength);
	
	if strength == 10 then
		strength = 500; -- 100 ton
	end;
	
	Player.MAX_PICKABLE_OBJECT_MASS = self.playerStuff.mass.old * strength;
end;

function PLAYER:setPlayerDistance(distance)
	Player.MAX_PICKABLE_OBJECT_DISTANCE = distance;
end;

function PLAYER:setChainsawUsage(state)
	g_currentMission.chainsaw_adminOnly = state;
end;

function PLAYER:createFarmerThouchOverlay(i)
	local offsetY 	= self.multiplyHudsOffset[2] * (i - 1);
	local uiScale 	= GameExtension.getSetting("PLAYER", "FARMERS_TOUCH_HUD_SCALE") * g_gameSettings:getValue("uiScale");
	local position 	= {getNormalizedScreenValues(GameExtension.getSetting("PLAYER", "FARMERS_TOUCH_POS_X"), GameExtension.getSetting("PLAYER", "FARMERS_TOUCH_POS_Y"))};
	local size 		= {getNormalizedScreenValues(512 * uiScale, 44 * uiScale)};
	local overlay1 = Overlay:new("farmersThouchOverlay1" .. i, GameExtension.hudElements, GameExtension.inverseValue(position[1], size[1]), position[2] + offsetY, size[1], size[2]);
	overlay1:setUVs(GameExtension.normalizeUVs({0, 1, 512, 45}));
	overlay1:setColor(unpack(GameExtension.colorCodes["BG_GREY_1"]));
	
	local overlay2 = Overlay:new("farmersThouchOverlay2" .. i, GameExtension.hudElements, GameExtension.inverseValue(position[1], size[1]), position[2] + offsetY, size[1], size[2]);
	overlay2:setUVs(GameExtension.normalizeUVs({0, 46, 512, 90}));
	overlay2:setColor(unpack(GameExtension.colorCodes["BG_BRIGHT_BLACK"]));
	
	table.insert(self.farmersThouchOverlays, {overlay1, overlay2});
end;

function PLAYER:emptyFarmersTouch()
	for i = 2, #self.showGrowthStateAt do
		self.showGrowthStateAt[i] = nil;
		
		if self.farmersThouchOverlays[i] ~= nil then
			self.farmersThouchOverlays[i][1]:delete();
			self.farmersThouchOverlays[i][2]:delete();
			self.farmersThouchOverlays[i] = nil;
		end;
	end;
end;

function PLAYER.setToolById(self, oldFunc, toolId, noEventSend)
	if not g_currentMission.chainsaw_adminOnly then
		oldFunc(self, toolId, noEventSend);
	else
		local MP = g_currentMission.missionDynamicInfo.isMultiplayer;
		if g_currentMission.isMasterUser or self.isServer or not MP then
			oldFunc(self, toolId, noEventSend);
		else
			g_currentMission:showBlinkingWarning(g_i18n:getText("CHAINSAW_ADMIN_WARNING"), 1000);
			
			oldFunc(self, 0, true); -- We shouldn't had the time to select the chainsaw already, send no event for this reason.
		end;
	end;
end;
GameExtension.addClassOverride("OVERRIDE_SET_TOOL_BY_ID", Player, "setToolById", PLAYER.setToolById);


function PLAYER.pickUpObject(self, oldFunc, ...)
	if self.lastFoundObject ~= nil then
		local oldMass;
		if self.isServer and entityExists(self.lastFoundObject) then
			oldMass = getMass(self.lastFoundObject);
			setMass(self.lastFoundObject, oldMass * GameExtension.getSetting("PLAYER", "PLAYER_JOINT_STRENGTH"));
		end;
		
		oldFunc(self, ...);
		
		if self.isServer and oldMass ~= nil then
			setMass(self.lastFoundObject, oldMass);
		end;
	end;
end;
GameExtension.addClassOverride("OVERRIDE_OBJECT_LIFTING", Player, "pickUpObject", PLAYER.pickUpObject);