--
-- TIME
--
-- @author:    	Xentro (Marcus@Xentro.se)
-- @website:	www.Xentro.se
-- @history:	v1.0 - 2016-11-29 - Initial implementation
-- 

TIME = {};

function TIME:loadMap()
	self.increaseTimeScale = {};
	self.increaseTimeScale.start = GameExtension.getSetting("TIME", "FASTFORWARD_NIGHT_START");
	self.increaseTimeScale.stop  = GameExtension.getSetting("TIME", "FASTFORWARD_NIGHT_STOP");
	
	GameExtension.disableModClass("MapAutosave_LS17", "MapAutosave");
end;

function TIME:update(dt)
	if g_currentMission:getIsServer() or g_currentMission.isMasterUser then
		if self:getIsActiveForInput() and self.oldTimeScale ~= nil then
			if InputBinding.hasEvent(InputBinding.INCREASE_TIMESCALE) then
				TIME.setNewTimeScale(self, 1);
			end;
			
			if InputBinding.hasEvent(InputBinding.DECREASE_TIMESCALE) then
				TIME.setNewTimeScale(self, -1);
			end;
		end;
	end;
end;	

function TIME:updateTick(dt)
	if g_currentMission:getIsServer() or g_currentMission.isMasterUser then -- Admins need to know that they cant change the timeScale now..
		local override = false;
		
		if g_modIsLoaded["ZZZ_fastForward"] then
			override = _G["ZZZ_fastForward"]["fastForward"].Active;
		end;
		
		if not override then
			if g_currentMission.environment.currentHour >= self.increaseTimeScale.start or g_currentMission.environment.currentHour < self.increaseTimeScale.stop then
				local newTimeScale = GameExtension.getSetting("TIME", "FASTFORWARD_NIGHT_SCALE");
				
				if newTimeScale > 0 and self.oldTimeScale == nil then
					self.oldTimeScale = g_currentMission.missionInfo.timeScale;
					TIME.setTimeScale(self, newTimeScale); -- Set our new timeScale
				end;
			else
				if self.oldTimeScale ~= nil then
					TIME.setTimeScale(self, 0); -- Reset to our old timeScale
				end;
			end;
		end;
	end;
end;

function TIME:setNewTimeScale(value)
	local settings = GameExtension.getSetting("TIME", nil, true)["FASTFORWARD_NIGHT_SCALE"];
	
	value = Utils.clamp(settings.value + value, settings.limits.min + 1, settings.limits.max); -- set min to 1 as 0 disable the feature
	
	if value ~= settings.value then
		GameExtension.setSetting("TIME", "FASTFORWARD_NIGHT_SCALE", value);
	end;
end;

function TIME:setTimeScale(value)
	-- Only admins or server is getting trough
	if self.oldTimeScale ~= nil then
		local oldTimeScale = self.oldTimeScale;
		self.oldTimeScale = nil;
		
		-- Only let server set timeScale
		if g_currentMission:getIsServer() then
			local timeScale = value;
			if timeScale == 0 then
				timeScale = oldTimeScale;
			else
				if timeScale == 1 then
					timeScale = 1.2;
				end;
				
				timeScale = timeScale * 100;
			end;
		
			g_currentMission:setTimeScale(timeScale);
		end;
		
		if value > 0 then
			self.oldTimeScale = oldTimeScale;
		end;
	end;
end;

function TIME:setSaveInterval(value)
	-- addMessage("autoSave", g_currentMission.missionDynamicInfo);
	-- addMessage("autoSaveInterval", g_currentMission);
	-- addMessage("autoSaveTime", g_currentMission);
	-- addMessage("time", g_currentMission);
	
	if g_currentMission:getIsServer() and g_dedicatedServerInfo == nil then
		local oldTime;
		local newTime = value * 60 * 1000; -- minutes
		
		if g_dedicatedServerInfo ~= nil then
			oldTime = g_dedicatedServerInfo.autoSaveInterval;
			g_dedicatedServerInfo.autoSaveInterval = newTime;
		else
			oldTime = g_currentMission.autoSaveInterval;
			g_currentMission.autoSaveInterval = newTime;
		end;
		
		g_currentMission.autoSaveTime = (g_currentMission.autoSaveTime - oldTime) + newTime;
		
		-- Enable the Auto Save option in SP, should we do this for MP too? there is an setting for this in MP though so...
		if not g_currentMission.missionDynamicInfo.isMultiplayer then
			g_currentMission.missionDynamicInfo.autoSave = value > 0;
		end;
	end;
end;

function TIME:newSetTimeScale(oldFunc, value, noEventSend)
	if g_currentMission.GameExtension ~= nil and g_currentMission.GameExtension.oldTimeScale ~= nil and not noEventSend then
		-- We let admins toggle "timeScale" trough our inputbinding
		
		-- work around for this mod
		if g_modIsLoaded["ZZZ_fastForward"] then
			local mod = _G["ZZZ_fastForward"]["fastForward"];
			
			if mod.Active or not mod.Active and mod.oldTimeScale ~= nil and mod.oldTimeScale == value then
				if not mod.Active then
					mod.oldTimeScale = nil;
				end;
				
				oldFunc(self, value, noEventSend);
			end;
		end;
	else
		oldFunc(self, value, noEventSend);
	end;
end;
-- FSBaseMission.setTimeScale = Utils.overwrittenFunction(FSBaseMission.setTimeScale, TIME.newSetTimeScale);
GameExtension.addClassOverride("OVERRIDE_TIME_SCALE", FSBaseMission, "setTimeScale", TIME.newSetTimeScale);