--
-- VEHICLE
--
-- @author:    	Xentro (Marcus@Xentro.se)
-- @website:	www.Xentro.se
-- @history:	v1.0 - 2016-10-30 - Initial implementation
-- 

VEHICLE = {};

function VEHICLE:loadMap()
	GameExtension.addSpecialization("VEHICLE_SPEC", VEHICLE_SPEC);
	
	local posX = GameExtension.getSetting("VEHICLE", "VEHICLE_HUD_POS_X");
	-- Lets do this not to mess up with any custom values 
	if GameExtension.getSetting("VEHICLE", "VEHICLE_HUD_SCALING") then
		posX = posX + (227 * (g_gameSettings:getValue("uiScale") - 0.5));
	end;
	
	local uiScale 	= GameExtension.getSetting("VEHICLE", "VEHICLE_HUD_SCALE") * g_gameSettings:getValue("uiScale");
	local position 	= {getNormalizedScreenValues(posX, GameExtension.getSetting("VEHICLE", "VEHICLE_HUD_POS_Y"))};
	local size 		= {getNormalizedScreenValues(38 * uiScale, 183 * uiScale)};
	
	self.vehicleHudOverlay = Overlay:new("vehicleHudOverlay", GameExtension.hudElements, GameExtension.inverseValue(position[1], size[1]), position[2], size[1], size[2]);
	self.vehicleHudOverlay:setUVs(GameExtension.normalizeUVs({473, 91, 511, 274}));
	self.vehicleHudOverlay:setColor(unpack(GameExtension.colorCodes["BG_GREY_1"]));
	
	self.iconOffset = {getNormalizedScreenValues(4 * uiScale, 5 * uiScale)};
	
	-- if GameExtension.MESSAGE_MODE == GameExtension.DEBUG then
		-- self.vehicleHudPos = VEHICLE.vehicleHudPos;
		-- addConsoleCommand("vehicleHudPos", "vehicleHudPos", "vehicleHudPos", self);
	-- end;
end;

function VEHICLE:deleteMap()
	if self.vehicleHudOverlay ~= nil then
		self.vehicleHudOverlay:delete();
	end;
end;
 
function VEHICLE:update(dt)
end;

function VEHICLE:updateTick(dt)
	if g_currentMission:getIsClient() then
		if self.vehicleHudOverlay ~= nil then
			if g_currentMission.showVehicleInfo and g_currentMission.controlledVehicle ~= nil and GameExtension.getSetting("VEHICLE", "VEHICLE_HUD_ACTIVE") then
				local icons = {};
				
				-- TODO: Reduce to min needed calls..
				if g_currentMission.controlledVehicle.getVisualAidIcons ~= nil then
					g_currentMission.controlledVehicle:getVisualAidIcons(icons);
					
					if #icons > 0 then
						self.currentHudIcons = icons;
					else
						self.currentHudIcons = nil;
					end;
				end;
			else
				self.currentHudIcons = nil;
			end;
		end;
	end;
end;

function VEHICLE:draw()
	if g_currentMission:getIsClient() then
		if self.vehicleHudOverlay ~= nil then
			if g_currentMission.showVehicleInfo and g_currentMission.controlledVehicle ~= nil and self.currentHudIcons ~= nil then
				self.vehicleHudOverlay:render();
				
				for i, v in ipairs(self.currentHudIcons) do
					if i > 8 then break; end; -- Stop, hud icon limit
					
					local offsetY = self.iconOffset[2] + ((v.icons[1].height * 1.1) * (i - 1)); -- Icon size must be the same for all!
					local id = 1;
					
					if v.type == "ENGINE" then
						if v.vehicle.isMotorStarted then
							id = 3;
						end;
						-- TODO: pre heat
						
					elseif v.type == "TIPPING_SIDE" then
						id = v.vehicle.preferedTipReferencePointIndex;
						
						if v.vehicle.tipState ~= Trailer.TIPSTATE_CLOSED then
							id = v.vehicle.numTipReferencePoints + 1; -- tipping icon is the last icon
						end;
						
					elseif v.type == "BALER" then
						if v.vehicle.baler.unloadingState == Baler.UNLOADING_CLOSED then
							-- if v.vehicle:isUnloadingAllowed() then
								-- if #v.vehicle.baler.bales > 0 then
									-- id = 2; -- Ready to unload
								-- end;
							-- end;
						else
							id = 1; -- Unloading
						end;
					end;
					
					if v.icons[id] ~= nil then
						v.icons[id].x = self.vehicleHudOverlay.x + self.iconOffset[1];
						v.icons[id].y = self.vehicleHudOverlay.y + offsetY;
						v.icons[id]:render();
					end;
				end;
			end;
		end;
	end;
end;

function VEHICLE:setTabState(state)
	g_currentMission.isToggleVehicleAllowed = state;
end;

function VEHICLE:vehicleHudPos(value)
	local uiScale 	= GameExtension.getSetting("VEHICLE", "VEHICLE_HUD_SCALE") * g_gameSettings:getValue("uiScale");
	-- local x, y 		= getNormalizedScreenValues(value, value * uiScale);
	local x, y 		= getNormalizedScreenValues(140 + (value * (g_gameSettings:getValue("uiScale") - 0.5)), 25);
	
	self.vehicleHudOverlay.x = GameExtension.inverseValue(x, self.vehicleHudOverlay.width);
	-- self.vehicleHudOverlay.y = y;
	-- self.iconOffset = {getNormalizedScreenValues(4 * uiScale, value * uiScale)};
	
	print("vehicleHudPos X: " .. tostring(value));
end;



-- Fix, particles will activate if your "tipping" to ground
-- Issue: Overloading.lua, line 238, Patch 1.3.1.0
function VEHICLE:newUpdateTick(oldFunc, dt)
	oldFunc(self, dt);
	
	if self.sentOurPipeFixUpdate == nil then
		self.sentOurPipeFixUpdate = false;
	end;
	
	if self.isServer then
		if self.targetTrailer == nil and self.shovelTargetFound == nil then
			if self.dischargeToGround then
				if self.overloading.didOverload or self.sentOurPipeFixUpdate ~= self.overloading.didOverload then
					self.sentOurPipeFixUpdate = self.overloading.didOverload;
					self:raiseDirtyFlags(self.overloadingDirtyFlag);
				end;
			end;
		end;
	end;
	
	-- showMessage("didOverload", tostring(self.overloading.didOverload));
	-- showMessage("isActive", tostring(self.overloading.isActive));
end;
-- Pipe.updateTick = Utils.overwrittenFunction(Pipe.updateTick, VEHICLE.newUpdateTick);
GameExtension.addClassOverride("OVERRIDE_FIX_PARTICLE", Pipe, "updateTick", VEHICLE.newUpdateTick);



VEHICLE_SPEC = {};

function VEHICLE_SPEC:load(savegame)
	self.getVisualAidIcons = VEHICLE_SPEC.getVisualAidIcons;
	
	self.gameExtension.hudIcons = {};
	
	if self.isClient then
		if SpecializationUtil.hasSpecialization(Motorized, self.specializations) then
			local huds = {};
			
			table.insert(huds, GameExtension.loadHudIcon("engineOff", {0,  91, 47,  123}));
			table.insert(huds, GameExtension.loadHudIcon("enginePre", {47, 91, 94,  123}, "ICON_ORANGE"));
			table.insert(huds, GameExtension.loadHudIcon("engineOn",  {94, 91, 141, 123}, "ICON_GREEN"));
		
			table.insert(self.gameExtension.hudIcons, {name = "ENGINE", isActive = true, icons = huds});
		end;
		
		if SpecializationUtil.hasSpecialization(Trailer, self.specializations) then
			if self.numTipReferencePoints > 1 then
				local huds = {};
				local txt = "";
				
				-- We need to arrange icons according to self.preferedTipReferencePointIndex
				for i, v in ipairs(self.tipAnimations) do
					local name = tostring(v.name):lower(); -- Use this as last resort
					
					if v.animationName ~= nil then
						name = v.animationName:lower(); -- Use this 2nd
					end;
					
					if v.doorAnimationName ~= nil then
						name = v.doorAnimationName:lower(); -- Use this 1th
					end;
					
					if name:find("grain") ~= nil and name:find("door") ~= nil then
						table.insert(huds, GameExtension.loadHudIcon("tip_backDoor",  {282, 91, 329, 123}));
						txt = txt .. " GrainDoor";
					elseif name:find("back") ~= nil then
						table.insert(huds, GameExtension.loadHudIcon("tip_back",  {235, 91, 282, 123}));
						txt = txt .. " Back";
					elseif name:find("left") ~= nil then
						table.insert(huds, GameExtension.loadHudIcon("tip_left",  {141, 91, 188, 123}));
						txt = txt .. " Left";
					elseif name:find("right") ~= nil then
						table.insert(huds, GameExtension.loadHudIcon("tip_right", {188, 91, 235, 123}));
						txt = txt .. " Right";
					end;
					
					txt = txt .. " (" .. name .. "),";
				end;
				
				table.insert(huds, GameExtension.loadHudIcon("tipping", {329, 91, 375, 123}, "ICON_RED"));
				
				-- log("DEBUG", "Found tipSides:" .. txt .. " - " .. self.configFileName);
				
				if #huds > 0 then
					table.insert(self.gameExtension.hudIcons, {name = "TIPPING_SIDE", isActive = true, icons = huds});
				end;
			end;
		end;
		
		if SpecializationUtil.hasSpecialization(Baler, self.specializations) then
			local huds = {};
			
			table.insert(huds, GameExtension.loadHudIcon("unloadingBaler", {375, 91, 422, 123}, "ICON_RED"));
			-- table.insert(huds, GameExtension.loadHudIcon("readyToUnload",  {375, 91, 422, 123}, "ICON_GREEN"));
		
			table.insert(self.gameExtension.hudIcons, {name = "BALER", isActive = true, icons = huds});
		end;
	end;
end;

function VEHICLE_SPEC:delete()
	if self.isClient then
		for _, v in ipairs(self.gameExtension.hudIcons) do
			for _, i in ipairs(v.icons) do
				i:delete();
			end;
		end;
	end;
end;

function VEHICLE_SPEC:update(dt)
	if self.isClient and g_currentMission.controlledVehicle == self then
		if self.checkDcUnit ~= nil then
			local state = GameExtension.getSetting("VEHICLE", "DC_USE_SHUTTLE");
			
			if state > 0 and state ~= self.driveControl.shuttle.isActive then
				-- log("DEBUG", "DC change " .. tostring(state));
				self.driveControl.shuttle.isActive = state == 1;
				
				local modName = "ZZZ_driveControl";
				if g_modIsLoaded[modName] then
					_G[modName]["driveControlInputEvent"].sendEvent(self);
					-- driveControlInputEvent.sendEvent(self); -- Does the same as above and the only way we can call that event from another mod!
				else
					-- Someone had changed the name, maybe good to show an error?
					-- log("ERROR", "");
				end;
			end;
		end;
		
		if self.checkGearboxUnit ~= nil then
			local state = GameExtension.getSetting("VEHICLE", "GB_USE_GEARBOX");
			
			if state > 0 and state ~= self.mrGbMS.IsOnOff then
				self:mrGbMSetIsOnOff(state == 1);
			end;
		end;
	end;
	
	self.checkDcUnit = nil;
	self.checkGearboxUnit = nil;
end;

function VEHICLE_SPEC:onEnter(isControlling)
	if self.dCcheckModule ~= nil then
		self.checkDcUnit = true;
	end;
	
	if self.mrGbMS ~= nil then
		self.checkGearboxUnit = true;
	end;
end;

function VEHICLE_SPEC:getVisualAidIcons(hudIcons)
	for i, v in ipairs(self.gameExtension.hudIcons) do
		local add = true;
		
		if v.name == "ENGINE" then
			add = not g_currentMission.missionInfo.automaticMotorStartEnabled;
		end;
		
		if v.isActive and add then
			table.insert(hudIcons, {type = v.name, vehicle = self, index = i, icons = v.icons});
		end;
	end;
	
	for _, implement in pairs(self.attachedImplements) do
		if implement.object ~= nil and implement.object.getVisualAidIcons ~= nil then
			implement.object:getVisualAidIcons(hudIcons);
		end;
	end;
end;



-- Written and included on the advice of Decker_MMIV
local injectFillLevelInfo =
function(self, superFunc, fillLevelInformations)
    superFunc(self, fillLevelInformations)
	
    g_currentMission.mod_fillLevelInformations = fillLevelInformations
    g_currentMission.mod_fillLevelInformationIdx = 0
end

-- Vehicle.getFillLevelInformation = Utils.overwrittenFunction(
    -- Vehicle.getFillLevelInformation, injectFillLevelInfo
-- )

-- RailroadVehicle.getFillLevelInformation = Utils.overwrittenFunction(
    -- RailroadVehicle.getFillLevelInformation, injectFillLevelInfo
-- )

-- local origDrawLevelBar = FSBaseMission.drawLevelBar

-- FSBaseMission.drawLevelBar = function(self, overlay1, pct, text1, text1color, text2, text2color, overlay2)
local injectDrawLevelBar = 
function(self, origDrawLevelBar, overlay1, pct, text1, text1color, text2, text2color, overlay2)
    if  overlay2 ~= g_currentMission.fuelLevelIconOverlay
    and g_currentMission.mod_fillLevelInformations ~= nil
    and #g_currentMission.mod_fillLevelInformations > g_currentMission.mod_fillLevelInformationIdx
    then
        g_currentMission.mod_fillLevelInformationIdx = g_currentMission.mod_fillLevelInformationIdx + 1
        if pct > 0 then
            if GameExtension.getSetting("VEHICLE", "FILLTYPE_ALA_DECKER") then
				local t = GameExtension.getSetting("VEHICLE", "FILLTYPE_SHOW_DURATION");
				local dt = g_currentMission.time % t
				if dt >= (t * 0.6) then
					local fillType = g_currentMission.mod_fillLevelInformations[ g_currentMission.mod_fillLevelInformationIdx ].fillType
					local fillDesc = FillUtil.fillTypeIndexToDesc[ fillType ]
					if fillDesc ~= nil then
						local factor = 1
						if dt >= t * 0.9 then
							factor = math.min(1, (dt - (t * 0.9)) / (t * 0.1))
						else
							factor = math.max(0, ((t * 0.7) - dt) / (t * 0.1))
						end
						text1color[4] = text1color[4] * factor
						text2color[4] = text2color[4] * factor
					   
						setTextBold(false)
						setTextAlignment(RenderText.ALIGN_LEFT);
						setTextColor(overlay1.overlayValue.r,overlay1.overlayValue.g,overlay1.overlayValue.b,overlay1.overlayValue.a * (1 - factor));
						renderText(overlay1.x, overlay1.y + g_currentMission.levelTextOffsetY, g_currentMission.levelTextTextSize, tostring(fillDesc.nameI18N));
					end
				end
            end
        end
    end

    origDrawLevelBar(self, overlay1, pct, text1, text1color, text2, text2color, overlay2)
end

GameExtension.addClassOverride("OVERRIDE_VEHICLE_FILLLEVEL", Vehicle, "getFillLevelInformation", injectFillLevelInfo);
GameExtension.addClassOverride("OVERRIDE_TRAIN_FILLLEVEL", RailroadVehicle, "getFillLevelInformation", injectFillLevelInfo);
GameExtension.addClassOverride("OVERRIDE_DRAW_LEVEL_BAR", FSBaseMission, "drawLevelBar", injectDrawLevelBar);